from django.conf import settings
from django.utils import timezone

from Crypto.Hash import SHA512
from zeep import Client

from . import exceptions


class PayCometClient:
    wsdl = "https://api.paycomet.com/gateway/xml-bankstore?wsdl"

    def __init__(self):
        self.client = Client(wsdl=self.wsdl)
        self.merchant_code = settings.PAYCOMET_MERCHANT_CODE
        self.terminal = settings.PAYCOMET_TERMINAL
        self.password = settings.PAYCOMET_PASSWORD
        self.jet_id = settings.PAYCOMET_JET_ID

    def create_payment(self, request, pay_tpv_token, amount):
        ip = self._get_client_ip(request)
        id_user, token_user = self.add_user_token(pay_tpv_token, ip)
        return self.execute_purchase(id_user, token_user, amount, ip)

    def add_user_token(self, pay_tpv_token, ip):
        signature = self._get_sha512_signature(
            f"{self.merchant_code}{pay_tpv_token}{self.jet_id}{self.terminal}{self.password}"
        )

        params = {
            'DS_MERCHANT_MERCHANTCODE': self.merchant_code,
            'DS_MERCHANT_TERMINAL': self.terminal,
            'DS_MERCHANT_JETTOKEN': pay_tpv_token,
            'DS_MERCHANT_JETID': self.jet_id,
            'DS_MERCHANT_MERCHANTSIGNATURE': signature,
            'DS_ORIGINAL_IP': ip,
        }

        response = self.client.service.add_user_token(**params)

        self._check_response_error(response)

        return response['DS_IDUSER'], response['DS_TOKEN_USER']

    def execute_purchase(self, id_user, token_user, amount, ip):
        date = timezone.now().strftime("%Y/%m/%d %H:%M:%S")
        order = f"MERCHANTORDER-{date}"
        signature = self._get_sha512_signature(
            f"{self.merchant_code}{id_user}{token_user}"
            f"{self.terminal}{amount}{order}{self.password}"
        )

        product_description = "XML_BANKSTORE TEST product_description"
        owner = "XML_BANKSTORE TEST owner"

        params = {
            'DS_MERCHANT_MERCHANTCODE': self.merchant_code,
            'DS_MERCHANT_TERMINAL': self.terminal,
            'DS_IDUSER': id_user,
            'DS_TOKEN_USER': token_user,
            'DS_MERCHANT_AMOUNT': amount,
            'DS_MERCHANT_ORDER': order,
            'DS_MERCHANT_CURRENCY': 'EUR',
            'DS_MERCHANT_MERCHANTSIGNATURE': signature,
            'DS_ORIGINAL_IP': ip,
            'DS_MERCHANT_PRODUCTDESCRIPTION': product_description,
            'DS_MERCHANT_OWNER': owner,
            'DS_MERCHANT_SCORING': '',
            'DS_MERCHANT_DATA': '',
            'DS_MERCHANT_MERCHANTDESCRIPTOR': '',
            'DS_MERCHANT_SCA_EXCEPTION': '',
            'DS_MERCHANT_TRX_TYPE': '',
            'DS_ESCROW_TARGETS': '',
            'DS_USER_INTERACTION': '',
        }

        response = self.client.service.execute_purchase(**params)

        self._check_response_error(response)

        return response

    @staticmethod
    def _get_client_ip(request):
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            return x_forwarded_for.split(',')[0]
        return request.META.get('REMOTE_ADDR')

    @staticmethod
    def _get_sha512_signature(code):
        return SHA512.new(code.encode('utf-8')).hexdigest()

    @staticmethod
    def _check_response_error(response):
        error = response['DS_ERROR_ID']
        if error not in [0, '0']:
            raise exceptions.PaymentError(
                f"Se ha producido un error: Código {error}"
            )



from django.conf import settings
from django.views.generic import FormView

from . import services
from .exceptions import PaymentError
from .forms import PaymentForm
from .payment import PayCometClient


class PaymentView(FormView):
    form_class = PaymentForm

    def get_context_data(self, **kwargs):
        return {
            **super().get_context_data(**kwargs),
            'PAYCOMET_JET_ID': settings.PAYCOMET_JET_ID,
            'card_months': services.get_card_months(),
            'card_years': services.get_card_years(),
            'amount': self.get_amount(),
        }

    def get_amount(self):
        """
        Override this function to return the amount to be paid
        """
        raise NotImplementedError

    def form_valid(self, form):
        try:
            PayCometClient().create_payment(
                request=self.request,
                amount=form.cleaned_data['amount'],
                pay_tpv_token=form.cleaned_data['paytpvToken'],
            )
        except PaymentError as e:
            form.add_error(field=None, error=str(e))
            return self.form_invalid(form)
        else:
            self.actions_on_payment_success()
        return super().form_valid(form)

    def actions_on_payment_success(self):
        """
        Override this function if you want to send a confirmation email, for example
        """
        pass



from django import forms


class PaymentForm(forms.Form):
    """
    PAYCOMET form with JETID will return amount and paytpvToken.
    paytpvToken field does not follow PEP8, but we will rename it in the view.
    """

    amount = forms.DecimalField()
    paytpvToken = forms.CharField(max_length=64)



from django.utils import timezone


def get_card_months():
    months = []

    for x in range(1, 13):
        if x < 10:
            months.append(f'0{x}')
        else:
            months.append(str(x))

    return months


def get_card_years():
    """
    Returns current year plus 20
    """

    years = []
    current_year = timezone.now().year

    for x in range(current_year, current_year + 20):
        last_2_digits = str(x)[2:]
        years.append(last_2_digits)

    return years


def get_amount():
    return '0.01'



from django.apps import AppConfig
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


class PaycometConfig(AppConfig):
    name = 'paycomet'
    verbose_name = "PAYCOMET"

    def ready(self):
        required_settings = [
            'PAYCOMET_MERCHANT_CODE',
            'PAYCOMET_TERMINAL',
            'PAYCOMET_PASSWORD',
            'PAYCOMET_JET_ID',
        ]

        for key in required_settings:
            if not hasattr(settings, key):
                raise ImproperlyConfigured(f"{key} setting must not be empty.")
